from typing import List, Optional

from fastapi import APIRouter, Depends, HTTPException, status
from sqlalchemy.orm import Session

from app.db.session import get_db
from app.schemas.todo import Todo, TodoCreate, TodoUpdate
from app.crud import todo as todo_crud

router = APIRouter()


@router.get("/todos/", response_model=List[Todo])
def read_todos(
    skip: int = 0,
    limit: int = 100,
    db: Session = Depends(get_db)
):
    """
    Retrieve todos.
    """
    todos = todo_crud.get_todos(db, skip=skip, limit=limit)
    return todos


@router.post("/todos/", response_model=Todo, status_code=status.HTTP_201_CREATED)
def create_todo(
    todo: TodoCreate,
    db: Session = Depends(get_db)
):
    """
    Create new todo.
    """
    return todo_crud.create_todo(db=db, todo=todo)


@router.get("/todos/{todo_id}", response_model=Todo)
def read_todo(
    todo_id: int,
    db: Session = Depends(get_db)
):
    """
    Get todo by ID.
    """
    db_todo = todo_crud.get_todo(db, todo_id=todo_id)
    if db_todo is None:
        raise HTTPException(status_code=404, detail="Todo not found")
    return db_todo


@router.put("/todos/{todo_id}", response_model=Todo)
def update_todo(
    todo_id: int,
    todo: TodoUpdate,
    db: Session = Depends(get_db)
):
    """
    Update a todo.
    """
    db_todo = todo_crud.update_todo(db, todo_id=todo_id, todo=todo)
    if db_todo is None:
        raise HTTPException(status_code=404, detail="Todo not found")
    return db_todo


@router.delete("/todos/{todo_id}", status_code=status.HTTP_204_NO_CONTENT)
def delete_todo(
    todo_id: int,
    db: Session = Depends(get_db)
):
    """
    Delete a todo.
    """
    deleted = todo_crud.delete_todo(db, todo_id=todo_id)
    if not deleted:
        raise HTTPException(status_code=404, detail="Todo not found")
    return None